@@ -4,6 +4,7 @@ import json
 import logging
 import os
 import time
+from contextvars import ContextVar
 from copy import deepcopy
 from importlib import resources
 from typing import Any, Dict, Optional, Tuple
@@ -21,14 +22,16 @@ from multiauth.manager import User, UserManager
 from multiauth.providers.aws import aws_signature
 from multiauth.utils import setup_logger
 
+ctx_store: ContextVar['MultiAuth'] = ContextVar('multiauth_instance')
+
 
 def load_authrc(
     logger: logging.Logger,
-    authrc_file: Optional[str] = None,
+    authrc: Optional[str] = None,
 ) -> Tuple[Dict, Dict]:
     """Load authrc file."""
 
-    filepath = authrc_file or os.getenv('AUTHRC')
+    filepath = authrc or os.getenv('AUTHRC')
     if not filepath:
         if os.path.exists('.authrc'):
             filepath = '.authrc'
@@ -41,34 +44,57 @@ def load_authrc(
     logger.info(f'loading authrc file: {filepath}')
 
     with open(filepath, 'r', encoding='utf-8') as f:
-        authrc = json.load(f)
+        data = json.load(f)
 
-    if not 'auth' in authrc:
+    if not 'auth' in data:
         raise InvalidConfigurationError('auth section not found', path='$.auth')
 
-    if not 'users' in authrc:
+    if not 'users' in data:
         raise InvalidConfigurationError('users section not found', path='$.users')
 
-    return authrc['auth'], authrc['users']
+    return data['auth'], data['users']
 
 
 class MultiAuth(IMultiAuth):
 
     """Multiauth manager."""
 
+    def __new__(
+        cls,
+        auths: Optional[Dict] = None,
+        users: Optional[Dict] = None,
+        authrc: Optional[str] = None,
+        logger: Optional[logging.Logger] = None,  # pylint: disable=unused-argument
+    ) -> 'MultiAuth':
+        """Create a new instance of the Auth manager."""
+
+        try:
+            instance = ctx_store.get()
+        except LookupError:
+            instance = None
+
+        if instance is not None:
+            if instance.auths == auths and instance._manager.users == users and instance._authrc == authrc:
+                return instance
+
+        instance = super().__new__(cls)
+        ctx_store.set(instance)
+        return instance
+
     def __init__(
         self,
         auths: Optional[Dict] = None,
         users: Optional[Dict] = None,
-        authrc_file: Optional[str] = None,
+        authrc: Optional[str] = None,
         logger: Optional[logging.Logger] = None,
     ) -> None:
         """Initialize the Auth manager."""
 
         self._logger = logger or setup_logger()
+        self._authrc = authrc
 
         if auths is None or users is None:
-            auths, users = load_authrc(self._logger, authrc_file)
+            auths, users = load_authrc(self._logger, authrc)
 
         self.validate(auths, users)
 
@@ -82,6 +108,12 @@ class MultiAuth(IMultiAuth):
 
         return self._headers
 
+    @property
+    def auths(self) -> Dict[str, User]:
+        """Fetch all auths methods."""
+
+        return self._auths
+
     @property
     def users(self) -> Dict[str, User]:
         """Fetch all users of the internal manager."""
@@ -229,11 +261,22 @@ class MultiAuth(IMultiAuth):
     def reauthenticate(
         self,
         username: str,
-        headers: Optional[Dict[str, str]] = None,
+        additional_headers: Optional[Dict[str, str]] = None,
         no_auth: bool = False,
     ) -> Tuple[Dict[str, str], Optional[str]]:
-        """Reauthentication of the user in case of token expiry."""
+        """Reauthentication of the user in case of token expiry.
 
+        Args:
+            username: The username of the user to reauthenticate.
+            additional_headers: Additional headers to add to the returned headers.
+            no_auth: If True, do not authenticate the user.
+
+        Returns:
+            - The headers
+            - The username of the user, if authenticated.
+        """
+
+        headers = additional_headers or {}
         user_info = self._manager.users[username]
         expiry_time = user_info.expires_in
         refresh_token = user_info.refresh_token
@@ -264,7 +307,6 @@ class MultiAuth(IMultiAuth):
                 self._headers[username] = auth_response['headers']
                 self._logger.info('Reauthentication Successful')
 
-        headers = headers or {}
         if not no_auth:
             headers.update(self._headers.get(username, {}))
 
